#!/usr/bin/env node

'use strict';

var C = require('./lib/palette'),
    cli = require('cli').enable('glob'),
    path = require('path'),
    fse = require('fs-extra'),
    Case = require('case'),
    pluralize = require('pluralize'),
    banner = require('./lib/banner.js'),
    _ = require('lodash'),
    Task = {},
    Project = {},
    task_command, task_args;

Task.spawn = require('./lib/spawn_task');
Task.handle = require('./lib/handle_task');
Project.create = require('./lib/create');
Project.install = require('spawn-npm-install');
Project.generate = require('./lib/generate');
Project.refill = require('./lib/refill');
Project.upgrade = require('./lib/upgrade');

cli.parse({
    create: ['c', 'Generate resource from template. Ex: feds -c "My COOL Website" ./my_cool_website"', 'string'],
    generate: ['g', 'Generate resource from template. Ex: feds -g block menu common/main_menu', 'string'],
    refill: ['r', 'Install refill. Ex: feds -r slider common/full_screen_slider', 'string'],
    build: ['b', 'Build assets and resources for production enviroment.'],
    development: ['d', 'Watches files, builds assets and resources for development.'],
    install: ['i', 'Install dependencies.'],
    report: [null, 'Code quality test and report.'],
    docs: [null, 'Create docs from sources and _resource_.md files.'],
    upgrade: ['U', 'Upgrade boilerplate build tasks.'],
    verbose: [null, 'Debug.'],
    version: ['v', 'Project version']
});

var updateNotifier = require('update-notifier');
var pkg = require('./package.json');
updateNotifier({pkg: pkg}).notify();

cli.main(function (args, options) {
    // is any option set
    if (_.result(_.uniq(_.valuesIn(options)), 0) === null) {
        cli.fatal('Missing options. Use feds --help to list available commands');
        return;
    }

    /*================================================
    =           Non project functionality            =
    =================================================*/

    /*========== Create new project ==========*/
    if (options.version) {
        banner('CML cli info', '');
    }
    else if (typeof options.create === 'string') {
        if (args.length < 1) {
            cli.fatal('Create project requires 2 arguments. Ex: feds --create "Project name" ./target_directory');
            return;
        }
        var dest = path.join(process.cwd(), args[0]);

        banner('Creating project ' + options.create, dest);

        cli.info(C.key('Create project ') + C.value(options.create) + C.key(' into ') + C.value(dest));
        cli.spinner('Working...');
        Project.create({
            repository: args[1] || 'https://github.com/viktorbezdek/feds-boilerplate.git',
            destination: dest,
            name: Case.snake(options.create).replace('_', '-')
        }, function (err) {
            cli.spinner('Working...done', true);
            if (err) {
                cli.error('Error creating project...');
                cli.debug(C.error(err));
                return false;
            }
            cli.ok(C.success('Project created successfully...'));
            return true;
        });

        return;
    }


    /*================================================
    = In project functionality =
    ================================================*/
    else {

        if (!args) console.log('no args');

        try {
            // resolve project info
            Project.info = fse.readJsonSync(path.join(process.cwd(), 'feds.json'));
            // resolve project package.json
            Project.pkg = fse.readJsonSync(path.join(process.cwd(), 'package.json'));
        }
        catch (e) {
            cli.fatal('You are not in FEDS project folder. Can\'t find feds.json file.');
            return;
        }

        /*========== Refill resource project ==========*/

        if (typeof options.refill === 'string') {

            banner('Refilling resource ' + options.refill, args[0]);

            cli.info(C.key('Refill resource ') + C.value(options.refill) + C.key(' into ') + C.value(args[0]));
            cli.spinner('Working...');
            Project.refill({
                    repository: args[1] || 'https://github.com/viktorbezdek/feds-refill-' + Case.snake(options.refill).replace('_', '-'),
                    destination: path.join(process.cwd(), Project.info.sources.views),
                    subfolder: args[0] || null,
                    name: Case.pascal(options.refill)
                },
                function (err) {
                    cli.spinner('Working...done', true);
                    if (err) {
                        cli.error('Error creating project...');
                        cli.debug(C.error(err));
                        return false;
                    }
                    cli.ok(C.success('Resource refilled successfully...'));
                    return true;
                });

            return;
        }

        /*========== Install dependencies ==========*/

        if (options.install) {
            var join = function(sep) { return function(arr) { return arr.join(sep); }; };  // poor man's Ramda.join

            var packagesToInstall = _(Project.pkg.dependencies)
                .merge(Project.pkg.devDependencies)
                .pairs()
                .map(join('@'))
                .value();

            banner('Install project dependencies', process.cwd());

            cli.spinner('Installing...');
            Project.install(packagesToInstall, {
                stdio: 'inherit',
                cwd: process.cwd()
            }, function () {
                cli.spinner('Installing...done', true);
            });
        }

        /*========== Update build scripts ==========*/

        if (options.upgrade) {
            banner('Upgrade project core dependencies', process.cwd());
            cli.spinner('Upgrading...');
            Project.upgrade(null, function (err) {
                cli.spinner('Upgrading...done', true);
                if (err) {
                    cli.error('Error creating upgrading build scripts...');
                    cli.debug(C.error(err));
                    return false;
                }
                cli.ok(C.success('Build scripts upgraded successfully...'));
                return true;
            });
        }

        /*========== Production build ==========*/

        if (options.build) {
            task_args = Project.info.tasks.build.split(' ');
            task_command = task_args.shift();
            options.verbose && task_args.push('--silent');
            options.release && task_args.push('--release');
            banner('Build for production', process.cwd());
            cli.spinner('Working...');
            Task.handle(Task.spawn({
                command: task_command,
                arguments: task_args,
                cwd: process.cwd()
            }), function (err) {
                cli.spinner('Working...done', true);
                if (!err) {
                    cli.ok('Production build completed successfully.');
                    return;
                }

                cli.error(err);
            });
        }

        /*========== Development build ==========*/

        if (options.development) {
            task_args = Project.info.tasks.development.split(' ');
            task_command = task_args.shift();
            options.verbose && task_args.push('--silent');
            banner('Build for development', process.cwd());
            cli.spinner('Working...');
            Task.handle(Task.spawn({
                command: task_command,
                arguments: task_args,
                cwd: process.cwd()
            }), function (err) {
                cli.spinner('Working...done', true);
                if (!err) {
                    cli.ok('Development build completed successfully.');
                    return;
                }

                cli.error(err);
            });
        }

        /*========== Report ==========*/

        if (options.report) {
            task_args = Project.info.tasks.report.split(' ');
            task_command = task_args.shift();
            banner('Code quality test and report', process.cwd());
            cli.spinner('Working...');
            Task.handle(Task.spawn({
                command: task_command,
                arguments: task_args,
                cwd: process.cwd()
            }), function (err) {
                cli.spinner('Working...done', true);
                if (!err) {
                    cli.ok('Code quality test and report completed successfully.');
                    return;
                }

                cli.error(err);
            });
        }

        /*========== Docs ==========*/

        if (options.docs) {
            task_args = Project.info.tasks.docs.split(' ');
            task_command = task_args.shift();
            banner('Create docs', process.cwd());
            cli.spinner('Working...');
            Task.handle(Task.spawn({
                command: task_command,
                arguments: task_args,
                cwd: process.cwd()
            }), function (err) {
                cli.spinner('Working...done', true);
                if (!err) {
                    cli.ok('Docs created successfully.');
                    return;
                }
                cli.error(err);
            });
        }

        /*========== Generate resource(s) ==========*/

        if (options.generate) {

            if (!Project.info.generators.hasOwnProperty(options.generate)) {
                cli.fatal('Generator ' + options.generate + ' not found in feds.json.');
                return;
            }

            var opts = {
                type: options.generate,
                files: Project.info.generators[options.generate].files,
                destRoot: Project.info.sources[pluralize(options.generate)],
                names: args
            };

            cli.spinner('Working...');
            Project.generate(opts, function (err) {
                cli.spinner('Working...done', true);
                if (!err) {
                    cli.ok('Resource(s) generated successfully.');
                    return;
                }
                cli.error(err);
            });
        }

    }
});
